// Module Name: maze.cpp
// Author:      Stephen Manley (smanley@nyx.net)
//
// Description:
//
// This class is the maze object. It is comprised of two
// planes representing a floor and ceiling, a set of 
// X and Y dimensions, a rotation about it's axis, and
// a display scaling factor.
//
///////////////////////////////////////////////////////////////////////////

#ifndef MAZE
#include "maze.h"
#endif

#include <GL/gl.h>
#include <GL/glut.h>
#include <iostream.h>
#include <fstream.h>


Maze::Maze()
{
	// Rotation, scale and translation information
	xRotation = 45.0;
	yRotation = 0.0;

	xScale = 1.2;
	yScale = 1.2;
	zScale = 1.2;

	xSize = 20;
	ySize = 20;

	// Wait until the tiles have been processed!
	mazeEnabled = 0;
}

Maze::~Maze()
{
}

// Loads the maze from disk
void Maze::load()
{
    fstream inFile("grid.dat", ios::in);
    
	if (!inFile)
		return;

    // Read in the grid dimensions
    inFile >> xSize;
    inFile >> ySize;
    colorArray = new double[xSize*ySize*3];

    double r,g,b;

	// Read in the colors of each tile (X*Y)
    for (int i=0; i<xSize*ySize*3; i++)
    {
        inFile >> r;
        colorArray[i] = r;
        
        inFile >> g;
		colorArray[(++i)] = g;

        inFile >> b;
		colorArray[(++i)] = b;
    }

    inFile.close();

}

// Sets the animation function
void Maze::animate(int degrees)
{
	yRotation += degrees;
}

// Draws the maze
void Maze::draw()
{
   // Clear the old display
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
   // Apply model transformations
   glPushMatrix();

   //glTranslated(0,0,-60);  
   //glRotated(yRotation, 0., 0., 1.);
   //glScaled(xScale, yScale, zScale)

   Wall* myWall = new Wall(); 

   // Draw the ceiling (0.01 to win roundoff with wall)
   drawCheckerPlane(0.01);

   int i = -1;
   int tilesize = 1;
   double z = 0; // Ceiling Plane
   double z2 = -5; // Floor Plane


   if (mazeEnabled)
   {
	   for (int j=0; j<1600; j++)
			mazeWalls[j]->draw();
   }

/* For generating the maze without hidden removal code
   // Draw the walls
   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
          double r = colorArray[(++i)]; 
          double b = colorArray[(++i)];
          double g = colorArray[(++i)];

		  // Depending on what color was selected in the 2D maze creator,
		  // draw it with an appropriately styled tile.
          if (r == 1)
		  {              
			  // Top and bottom
   		      myWall->drawWall3(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall3(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall3(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall3(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }
   		  
          if (g == 1)
		  {
			  // Top and bottom
   		      myWall->drawWall2(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall2(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall2(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall2(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }
          if (b == 1)
		  {
			  // Top and bottom
   		      myWall->drawWall1(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      myWall->drawWall1(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      myWall->drawWall1(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      myWall->drawWall1(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }

       }
   }
*/
   // Draw the ceiling
   drawCheckerPlane(-5.01);

   glPopMatrix();

   delete myWall;
}

// Draw a checkerboard plane to navigate on
void Maze::drawCheckerPlane(double z)
{
   int tilesize = 1;
   int xSize = 20;
   int ySize = 20;
   
   //double z = -20;

   glColor3f(1.0, 0.0, 0.0);

   int i = -1;

   glBegin(GL_QUADS);

   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
         	  if ((x % 2) && (y % 2))
		  	  	glColor3f(1.0, 0.0, 0.0);	
		  	  else
		  	    continue; //glColor3f(0.0, 0.0, 0.0);
          
   		      glVertex3d(x,y,z);
   		      glVertex3d(x+tilesize, y, z);
   		      glVertex3d(x+tilesize, y-tilesize, z);
   		      glVertex3d(x, y-tilesize, z); 

       }
   }

   glEnd();
}


// Draw a "Ceiling" or "Floor"
void Maze::drawPlane(double z)
{
   int tilesize = 1;
   int xSize = 20;
   int ySize = 20;
   
   //double z = -20;

   glColor3f(1.0, 0.0, 0.0);

   int i = -1;

   glBegin(GL_QUADS);
   
   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
          double r = colorArray[(++i)]; 
          double b = colorArray[(++i)];
          double g = colorArray[(++i)];

          if ((r == 1) || (g == 1) || (b == 1))
		  {
		  	  glColor3f(.5, .5, .5);	
          
   		      glVertex3d(x,y,z);
   		      glVertex3d(x+tilesize, y, z);
   		      glVertex3d(x+tilesize, y-tilesize, z);
   		      glVertex3d(x, y-tilesize, z); 

   		  }
       }
   }

   glEnd();
}

// Generate the maze tiles - making sure that we only draw the ones that
// actually need to be drawn! Since this code has a nasty problem, I'll
// use the checkerboard instead. You can see where I was going with it.
void Maze::generateMaze()
{

/*
	// Load the maze from disk
	load();

	// Allocate space for the maximum number of tiles 
	// This could be optimized!
	for (int i=0; i<1600; i++)
	{
		mazeWalls[i] = new Wall(0,0,0, 0,0,0, 0,0,0, 0,0,0);
		mazeWalls[i]->setActive(0);
	}

	// Enable tile drawing mode
	mazeEnabled = 1;

	// Determine how many tiles we need and set states
	double r, g, b;
	double rCheck, gCheck, bCheck;

    int needsTop = 1;
	int needsBottom = 1;
	int needsLeft = 1;
	int needsRight = 1;

	int xPos, yPos;
	int tileCount = 0;

	double z = 0; // Ceiling Plane
	double z2 = -5; // Floor Plane

	int i = 0;

   for (int x=-10; x<xSize-10; x++)
   {
       for (int y=-10; y<ySize-10; y++)
       {
          double r = colorArray[(++i)]; 
          double b = colorArray[(++i)];
          double g = colorArray[(++i)];

		  // Depending on what color was selected in the 2D maze creator,
		  // draw it with an appropriately styled tile.
          if (r == 1)
		  {              
			  // Top and bottom
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      mazeWalls[(i-1)]->setActive(1);
   		      mazeWalls[i++]->setCoords(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );
   		      mazeWalls[(i-1)]->setActive(1);

              // Sides
   		      mazeWalls[i++]->setCoords(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      mazeWalls[(i-1)]->setActive(1);
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		      mazeWalls[(i-1)]->setActive(1);
   		  }
   		  
          if (g == 1)
		  {
			  // Top and bottom
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      mazeWalls[i++]->setCoords(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      mazeWalls[i++]->setCoords(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }
          if (b == 1)
		  {
			  // Top and bottom
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x+1,y,z2, x+1,y,z );
   		      mazeWalls[i++]->setCoords(x,y-1,z, x,y-1,z2, x+1,y-1,z2, x+1,y-1,z );

              // Sides
   		      mazeWalls[i++]->setCoords(x+1,y,z, x+1,y,z2, x+1,y-1,z2, x+1,y-1,z);
   		      mazeWalls[i++]->setCoords(x,y,z, x,y,z2, x,y-1,z2, x,y-1,z);
   		  }

       }
   }
*/
}

void Maze::addScale(double x, double y, double z)
{
	xScale += x;
	yScale += y;
	zScale += z;
}

void Maze::addRotation(double x, double y, double z)
{
	xRotation += x;
	//yRotation += y;
	//xRotation += z;
}

void Maze::setRotation(double x, double y, double z)
{
	xRotation += x;
	yRotation += y;
	xRotation += z;
}


void Maze::setScale(double x, double y, double z)
{
	xScale = x;
	yScale = y;
	zScale = z;
}

void Maze::getRotation(double &x, double &y, double &z)
{
	x = xRotation;
	y = yRotation;
	z = zRotation;
}

void Maze::getScale(double &x, double &y, double &z)
{
	x = xScale;
	y = yScale;
	z = zScale;
}

void Maze::setAnimate(int aniFlag)
{
	animated = aniFlag;
}

